/* $Id: util.hpp 61 2006-07-31 21:15:25Z pjohnson $
  ------------------------------------------------------------------------
  This file is part of the PIIM package; please see:
    Johnson, PLF and Slatkin, M. 2006. "Inference of population
    genetic parameters in metagenomics: a clean look at messy data."
    Genome Research (in press).
 
  Copyright 2006 The Regents of the University of California.  All
  rights reserved.
 
  IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
  DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY
  OF SUCH DAMAGE.
 
  REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
  DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
  IS". REGENTS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
  UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 
  This software is distributed under the terms of Version 2 of the
  GNU General Public License as published by the Free Software
  Foundation.  If you have not received a copy of Version 2 of the
  GNU General Public License with this program, you may download the
  GPL from the following url: http://www.gnu.org/copyleft/gpl.html.

  ---------------------------------------------------------------------------
  Useful functions & type definitions
  ---------------------------------------------------------------------------
*/
#ifndef UTIL_HPP
#define UTIL_HPP

#include <iostream>
#include <sstream>
#include <stdexcept>
#include <map>
#include <vector>

using namespace std;
typedef map<char,char*> TOpts;

//---------------------------------------------------------------------------

template <typename TContents> class CCache {
public:
    typedef CCache<TContents> TThisType;
    inline const TContents& operator[] (unsigned int i) const {
        if (i >= m_List.size()  ||  !m_List[i].second) {
            throw exception();
        } else {
            return m_List[i].first;
        }
    }

    inline TContents& Set(unsigned int i) {
        if (i >= m_List.size()) {
            m_List.resize(i+1);
        }
        m_List[i].second = true;
        return m_List[i].first;
    }

    inline void Reset(void) {
        m_List.clear();
    }

private:
    typedef vector< pair<TContents,bool> > TList;
    TContents m_Null;
    TList m_List;
};

/*---------------------------------------------------------------------------*/

inline void x_ProcessOptions(int argc, char **argv, const char* params,
                      const char* required, TOpts &opts,
                      const char* usage) {
    int c;
    bool err = false;
    bool missingReqd = false;

    while ((c = getopt(argc, argv, params)) != -1) {
        if (c == '?') {
            err = true;
        } else {
            opts[c] = optarg;
        }
    }

    for (const char* scan = required;  *scan  &&  !err;  ++scan) {
        if (opts.find(*scan) == opts.end()) {
            missingReqd = true;
        }
    }

    if (err  ||  missingReqd) {
        cerr << "Usage:\n"
             << "   " << argv[0] << usage
             << endl;
        if (*required != '\0') { //i.e. we have required elements
            cerr << "NOTE: Parameters '" << required << "' are required."
                 << endl;
        }
        if (err) {
            throw runtime_error("Invalid arguments.");
        } else {
            throw runtime_error("Missing required argument.");
        }
    }
}

/*---------------------------------------------------------------------------*/
// PRE : integer / double
// POST: string representation (this func should really have been in spec)
inline string IntToString(int i) {
    ostringstream oss;
    oss << i << flush;
    return oss.str();
}
inline string DblToString(double d) {
    ostringstream oss;
    oss << d << flush;
    return oss.str();
}

#endif /*ifndef UTIL_HPP*/
